#ifndef PLOTWIDGET_H
#define PLOTWIDGET_H

#include <QPen>
#include <QGLWidget>
#include <QToolButton>

#include "types.h"

//Plot widget class. graphs w/ grid, labels and curves can be shown
//uses plot settings class to keep track of plot limits
//keeps index of position & active/inactive status wrt the main window
class PlotWidget : public QGLWidget
{
    Q_OBJECT
public:
    PlotWidget(QWidget *parent =0, int i=0, bool applyall=false); //constructor
    void setActive(); //sets the plot active
    void setDeactive(); //sets the plot deactive
    void changeIndex(int j); //changes index to j (wrt main window)
    void changeTimeIndex(int j); //changes time index
    QSize minimumSizeHint() const; //suggested min size
    QSize sizeHint() const; //general size specifications

    void refreshPixmap(); //redraws plot

    //Plot zoom/boundary functions
    QMap<int,QVector<double> > getZoomStack();
    plotLimits getPlotLimits(); //used by clone and updateControlDialogValues in mainWindow
    void setPlotSettings(const PlotSettings &settings); //imports plot settings in contructor
    void initializePlotBoundaries();//fits plot boundaries to initial data (max&mins)
    void setPlotBoundaries(plotLimits p, PlotSettings *settings = 0); //changes view without adding to zoomstack

    //data structure& modifier functions
    xytVector data;//data storage object- holds 'n' arrays of xy points (see types.h)
    QList<xytVector> curveMap;//holds space for more than 1 curve
    //void trim(int id); //clears the curve at time index 'id'
    void setDataVec(xytVector vec); //sets all data
    void addCurveMap(xytVector vec); //adds f(x,t) to list
    
    //various accessors
    int getIndex();
    int getTimeIndex();
    int getMaxTimeIndex();
    QString getName();
    int getMarkerColor();
    int getLineColor();
    int getMarkerSize();
    int getLineWidth();
    int getMarkerType();
    int getTimeDirection();
    int getBreakStatus();
    int getDisplayMode();
    int getFullScale();
    bool getGridShown();
    bool getMarkersShown();
    bool getLinesShown();

    //graph labels
    QString titleLabel;
    QString timeLabel; //displays actual time
    QString indexLabel; //displays time index
    QString lowerLimitLabels; //shows lower x,y values
    QString upperLimitLabels; //shows upper x,y values
    QString displayModeLabel; //shows display mode (zoom, auto, step etc)
    bool labelsVisible[7]; //Determines visibility.
    int labelColors[7]; //Determines label colors.
    int labelFormats[7]; //Determines label format (# decimals).

signals:
    void mouseMoved(QPointF pos, int i); //emits mouse position in plot coords
    void mouseClicked(QPoint pos);     //emits mouse position in widget pixel coords
    void needUpdateControlDialog(QString s); //will connect to updateControlDialog slot of main window
    void widgetSelected(int); //connects to addSelectedWidget(int) from xvsMainWindow
    void widgetDeselected(int);//connects to removeSelectedWidget(int) from xvsMainWindow
    void statusUpdate(QString);//connects to status bar.

protected:
    void paintEvent(QPaintEvent *); //reimplements paint event to create rubberband (drag to zoom) functionality
    void resizeEvent(QResizeEvent *event); //reimplements resize event to keep zoom buttons in correct positions during resize
    void mousePressEvent(QMouseEvent *event);//reimplemented to add rubberband zoom functionality
    void mouseMoveEvent(QMouseEvent *event);//reimplemented to add rubberband zoom functionality
    void mouseReleaseEvent(QMouseEvent *event);//reimplemented to add rubberband zoom functionality
    void keyPressEvent(QKeyEvent *event);//implements all keyboard shortcuts
    void wheelEvent(QWheelEvent *event);//allows for scrolling with mouse wheel

private:
    //initial setup functions
    void setupButtons(); //initializes buttons and connects them to actions
    void setupLabels();  //initializes labels

    //general member variables
    QString name; //title or name of window
    int index; //grid index for main window (where it is located wrt other plots)
    int displayMode;     // 0:zoom 1:step 2:animate 3:freeze.. see types.h
    bool isActive; //is active wrt mouse & keyboard
    bool isSelected; //is selected by mouse in selection mode
    bool isApplyAllActive; //apply to all property
    bool fullScale; //autoscale member var

    //display member variables
    int lineColor; //1:red, 2:green, 3:blue, 4:cyan, 5:magenta, 6:yellow
    int markerColor; //same refs as lineColor
    bool linesShown; //hide/show curve lines
    bool markersShown;//hide/show points
    int markerType; //1:circle 2:square, 3:triangle
    int markerSize; //1-6 pixels (~radius)
    int lineWidth; // 1-4 pixels
    bool tagByDx; //tags non-uniform section by color
    int offset; //tracks which level of amr is being plotted

    int Margin;

    //time related members
    int timeIndex; //current time index
    int timeDirection; // 0:forward 1:backward
    int breakStatus;// 0:off 1:on  ->when on this property will restrict the animation mode to 1 cycle
    QTimer *timer; //times animations

    //Painter objects
    QPen linePen;
    QPixmap pixmap; //pixel map on which to draw offscreen before copying to screen

    //Drawing functions
    void drawGrid(QPainter *painter); //draws grid and ticks & tick mark labels
    void drawCurves(QPainter *painter); //draws curves
    void drawLabels(QPainter*painter); //draws labels
    void updateMaxMinLabels(); //updates the maxmin upper and lower labels to current values
    void updateRubberBandRegion(); //redraws rubberband

    //zoom/boundary
    QVector<PlotSettings> zoomStack; //keeps record of zoom information to allow back &forward actions
    int curZoom; //current zoomstack index
    plotLimits getBoundingLims(QList<double> x, QList<double> y);
    bool rubberBandIsShown; //if rubberband zoom is in effect
    QRect rubberBandRect; //rubber band containing rectangle object
    QToolButton *forwardButton; //connects to forward zoom action
    QToolButton *backButton;    //connects to backward zoom action
    QToolButton *zoomInButton;  //connects to zoomin action (2x)
    QToolButton *zoomOutButton; //connects to zoomout action (2x)
    //general utility functions
    double getMax(QList<double> vect);
    double getMin(QList<double> vect);

    bool debug; //debug option

    //display functions: called by displayTypeHandler slot
    void setMarkerVisible(bool v);
    void setLinesVisible(bool v);

    //context menus & actions
    /*QMenu *stats_menu;
    QAction *display_brief_stats;
    QAction *display_full_stats;
    //fl_addtopup(stats_menu,"brief|full %F",display_stats);

    QMenu apply_control_menu;
    QAction toggle_apply_all_on;

    QMenu apply_menu;
        QMenu funcYMenu;
        QMenu funcXMenu;
        QAction absVal;
        QAction negativeVal;
        QAction powerI;
        QAction sqrt;
        QAction exp;
        QAction log;
        QAction log10;
        QAction sin;
        QAction cos;
        QAction tan;
        QAction asin;
        QAction acos;
        QAction atan;
        QAction sinh;
        QAction cosh;
        QAction tanh;

        QMenu xTransMenu;
        QAction pxydv_x_plus_c;
        QAction pxydv_x_times_c;
        QAction pxydv_x_dev_about_mean;
        QAction pxydv_x_to_x_plus_it_dx;
        QAction pxydv_x_to_x_plus_t;
        QAction pxydv_x_to_x_minus_t;
        QAction pxydv_interpolate_x_to_pick_x;
        QAction pxydv_x_to_pick_x;
        QAction pxydv_x_to_unit;
        QAction pxydv_x_to_unit_rescale_y;
        QAction pxydv_x_to_unit_rescale_shift_y;
        QAction pxydv_x_to_eps_times_sin_omega_x;
        QAction pxydv_x_to_scaled_x_power_p;
        QAction pxydv_x_to_scaled_x_power_p_gen;
        QAction pxydv_x_to_ordinal;
        QAction pxydv_x_to_dvumsh;
        QAction pxydv_x_to_dvumsh_local;
        QAction pxydv_x_to_dvgmsh;
        QAction pxydv_2_to_1_linear;

        QMenu yTransMenu;
        QAction pxydv_y_plus_c;
        QAction pxydv_y_times_c;
        QAction pxydv_y_dev_about_mean;
        QAction pxydv_y_to_unit;
        QAction pxydv_each_y_to_unit;
        QAction pxydv_y_threshold;
        QAction pxydv_y_to_x;
        QAction pxydv_y_to_y_minus_x;
        QAction pxydv_y_to_dx;
        QAction pxydv_y_to_dy;
        QAction pxydv_y_to_1m21;
        QAction pxydv_y_to_y_best_linear_fit;
        QAction pxydv_y_to_dev_from_best_linear_fit;
        QAction pxydv_y_time_average;
        QAction pxydv_y_t_y;

        QMenu xyTransMenu;
        QAction pxydv_xyun;
        QAction pxydv_xyun_symm;
        QAction pxydv_xy_to_dquad_input;
        QAction pxydv_xy_to_yx;
        QAction pxydv_xy_to_mxmy;
        QAction pxydv_xy_to_vw;
        QAction pxydv_xy_rotate;
        QAction pxydv_overlay;

        QMenu tTransMenu;
        QAction pxydv_t_plus_c;
        QAction pxydv_t_times_c;
        QAction pxydv_t_to_ordinal;
        QAction pxydv_t_to_unit_interval;
        QAction pxydv_cpt_to_tau;
        QAction pxydv_t_ln_t_minus_tstar;
        QAction pxydv_t_to_pick_t;
        QAction pxydv_t_to_pick_y0;
        QAction pxydv_t_to_pick_int_dt;
        QAction pxydv_t_interpolate_t2_x2;
        QAction pxydv_t_interpolate_t2_x4;

        QMenu xtTransMenu;
        QAction pxydv_xt_transpose;

        QMenu xytTransMenu;
        QAction pxydv_xytun;
        QAction pxydv_xytun_and_lintseg_fixed;


        QAction old_pxydv_xyun;
        QAction pxydv_xy_to_dquad;
        QAction pxydv_flow_field_rescale;
        QAction pxydv_t_to_y0;
        QAction pxydv_t_to_int_dt;



        QMenu dDTMenu;

        QMenu filterMenu;
        QAction everyMthAction;
        QAction thinAction;
        QAction linearSegAction;
        QAction linearSegGeneralAction;
        QAction twoFixedAction;
        QAction fourFixedAction;
        QAction twoGeneralAction;
        QAction fourGeneralAction;

        QMenu pulseMassageMenu;
        QMenu chebyFitMenu;

        QMenu periodicBCMenu;
        QAction imposeBCAction;
        QAction addBCAction;
        QAction deleteBCAction;
        QAction extendBCAction;

        QMenu fourierTransformMenu;
        QAction forwardFFTAction;
        QAction reverseFFTAction;
        QAction powerFFTAction;

        QMenu waveletTransformMenu;
        QAction daub4forwardAction;
        QAction daub4reverseAction;

        QMenu sphSymGravMenu;
        QMenu micellaneousMenu;
    //QActions here from applyXY
    QAction pxydv_x_log_c_plus_x;
    QAction pxydv_x_log_one_plus_x;
    QAction pxydv_dy_dx;
    QAction pxydv_dy_dx_mid;
    QAction pxydv_x_dy_dx;
    QAction pxydv_int_dx;
    QAction pxydv_rint_dx;
    QAction pxydv_x_times_y;
    QAction pxydv_y_divide_x;
    QAction pxydv_int_dx_divide_x;
    QAction pxydv_int_x_p_y_dx;
    QAction pxydv_x_p;
    QAction pxydv_x_p_times_y;
    QAction pxydv_y_p;

    QMenu apply1_history_menu;

    QMenu apply2_menu;

    QMenu apply3_menu;

    QMenu apply4_menu;
    QAction rdXdtAction;
    QAction rdXdtTermsAction;
    QAction rdXdtLinearTermsAction;
    QAction rdXdtNonlinearTermsAction;
    QAction rdYdtAction;
    QAction rdYdtTermsAction;
    QAction expansionAction;

    QMenu apply6_menu;
    QAction bcResidualAction;
    QAction momcResidualAction;

    QMenu applyn_menu;

    QMenu reduce_menu;*/

public slots:
    //zoom slots
    void forward();//moves one index forward in zoomstack to those PlotSettings
    void back(); //moves one index back in zoomstack to those PlotSettings
    void zoomIn();//zooms to half span in xy
    void zoomInX(double x);//zooms to half span in x
    void zoomInY(double y);//zooms to half span in y
    void zoomInXY(QPointF pos);//zooms to half span in xy
    void zoomOut();//zooms out to 2x span in xy

    //boundary view slots
    void setMaxX(QString x);
    void setMaxY(QString y);
    void setMinX(QString x);
    void setMinY(QString y);
    void optimizePlotBoundaries(); //fits plot boundaries to current data (max&mins)
    void resetPlotLimits(plotLimits p); //deals with control dialog operations

    //mode slots
    void changeDisplayMode(int dmode); // changes all the zoom connections needed when switching modes
    void toggleFullScale(int b);
    void setApplyAll(bool y);//modifier for ApplytoAll property

    //direct modifier slots
    void changeLabelShown(int label, bool shown);
    void changeLabelColor(int label, int color);
    void changeLabelFormat(int label, int format);
    void changeMarkerColor(int newColor);
    void changeMarkerType(int newType);
    void changeMarkerSize(int newSize);
    void changeLineColor(int newColor);
    void changeLineWidth(int newWidth);
    void changeName(QString newName); //changes name of plot
    void displayTypeHandler(int dtype); //determines if points &&|| lines are shown

    //time slots
    void animate(); //calls timePlusPlus or timeMinusMinus depending on timeDirection
    void startAnimate(); //starts timer
    void stopAnimate(); //stops timer
    void timePlusPlus(); //increases time index by one
    void timeMinusMinus(); //decreases time index by one
    void setTimeIndex(int t); //sets time index to 't' provided it is in the correct range
    void changeTimeDirection(int direction);
    void changeBreakStatus(int b);

    //mpeg slot
    void makeMpeg();
    void exportToImage();
};


#endif // PLOTWIDGET_H
