#include "xvsmainwindow.h"
#include <QFile>
#include <QFileDialog>
#include <QListWidget>
using namespace std;

/****************************CONSTRUCTOR*****************************/
xvsMainWindow::xvsMainWindow(QWidget *parent) : QMainWindow(parent)
{
    setupUi(this);

    //creates and connects all functions of control window
    controlDialog = new ControlWindow(this);
    controlDialog->hide();
    setupControlDialog();

    //setup toolbar buttons and icons
    cdButton->setIcon(QIcon(":/images/process.png"));
    connect(cdButton, SIGNAL(clicked()),controlDialog,SLOT(show()));

    openSdfButton->setIcon(QIcon(":/images/openSDF.png"));
    connect(openSdfButton,SIGNAL(clicked()),this,SLOT(openSdfFile()));

    openTextButton->setIcon(QIcon(":/images/openText.png"));
    connect(openTextButton,SIGNAL(clicked()),this,SLOT(openTextFile()));

    //creates menu actions & then menus
    createMenuActions();
    createMenus();

    //create plot actions
    createPlotActions();

    //initialize member variables
    ifuzz = 1e-10;
    allCheckBoxState= false;
    numPlotWidgets =0;
    activeWidgetIndex=0;
    gridSize =2;
    ctrl_spaceKills=true;
    isPlotMaximized = false;
    pt2Member = NULL;
    controlButtonDepressed = false;
    selectionMode = false;

    //application wide connections
    connect(this,SIGNAL(widgetChanged(QString)),this,SLOT(updateControlDialogValues(QString)) );
    connect(killAllButton,SIGNAL(clicked()),this,SLOT(killAllPlotWindows()));
    connect(this,SIGNAL(closeWindow()), qApp,SLOT(quit()) ); //creates a universal quit signal

    //creates context menu
    createContextActions();
    createContextMenus();

    //creates status bar for status updates
    createStatusBar();
    formulaLabel->setText("Welcome to XVS 2.0");
}
/**************************END CONSTRUCTOR***************************/

/**************************File  section***************************/
//text (hack) format
void xvsMainWindow::openTextFile()
{
    //gets file name from file dialog window
    QString fileName = QFileDialog::getOpenFileName(this,tr("Open Text"), QDir::currentPath(), tr("Text Files (*.txt *.dat)"));

    if (fileName.isEmpty()) {
        statusBar()->showMessage("Filename was empty.",2000);
        return;
    }

    //initializes QFile object- file name
    QFile file(fileName);

    //ends if file is not readable
    if (!file.open(QIODevice::ReadOnly | QIODevice::Text)) {
        statusBar()->showMessage("Text file unreadable. Check text format",3000);
        return;
    }

    /****begin reading file with QTextStream object****/
    QTextStream in(&file);

    //finds number of datasets contained in file
    int numDataSets;
    QString line = in.readLine();
    QStringList listLine = line.split(':',QString::SkipEmptyParts);
    QString checkString = "NumDatasets";
    if (checkString != listLine.at(0)) {
        statusBar()->showMessage("File format was incorrect. Make sure first line reads: NumDataSets:int",3000);
        return;
    }
    numDataSets = listLine.at(1).toInt();


    for (int z=0; z<numDataSets;z++)
    {
        //will store data from file here
        QList<QList<double> > ydata;
        QList<double> xcurve;
        QList<double> tcurve;

        line = in.readLine(); //blank line in format
        QString name = in.readLine(); //name of dataset
        int nt = in.readLine().toInt();//timesteps of dataset

        //coordinate values (x)
        line = in.readLine();
        listLine = line.split('\t',QString::SkipEmptyParts);
        for (int x=1;x<listLine.count();x++) { //iterator for xposition (columns)
            xcurve.append(listLine.at(x).toDouble());
        }

        for(int i=0;i<nt;i++)
        {
            line = in.readLine();
            listLine = line.split('\t',QString::SkipEmptyParts);

            tcurve.append(listLine.at(0).toDouble());

            QList<double> ycurve;
            for (int j=1;j<listLine.count();j++) { //iterator for xposition (columns)
                ycurve.append(listLine.at(j).toDouble());
            }
            ydata.append(ycurve); //add curve at timestep i
        }
        /****end input from file****/

        //create new plot window for data
        addPlotWidget();
        PlotWidget *w = arrayPlotWidgets[activeWidgetIndex];
        w->data.Y = ydata;
        w->data.setUniformX(xcurve,tcurve.length());
        w->data.T = tcurve;

        w->timeLabel = QString("T:%1").arg(w->data.T.at(0),0,'E',3);

        //initialize name label
        w->changeName(name);
        w->data.name = name;

        //index label
        w->indexLabel = QString("%1/ %2").arg(0).arg(nt-1);

        //setup maximum index value
        controlDialog->opsIndexLine->setMaximum(w->getMaxTimeIndex());
        controlDialog->viewIndexLine->setMaximum(w->getMaxTimeIndex());
        controlDialog->browseIndexLine->setMaximum(w->getMaxTimeIndex());
        controlDialog->opsIndexLine->setMinimum(0);
        controlDialog->viewIndexLine->setMinimum(0);
        controlDialog->browseIndexLine->setMinimum(0);

        //fits boundaries to data max&mins
        w->initializePlotBoundaries();
    }
    file.close();
    updateControlDialogValues("updateAll");

    //initialize filename w/o location
    QStringList curDir = fileName.split('/');
    QString n;
    n = curDir.last();
    statusBar()->showMessage(n+" opened successfully",3000);
}

//sdf format
FofX xvsMainWindow::readSdfFile(const char *filename, int tLevel)
{
    int tl=0; //tl == time level
    char  fn_sdf[100]="", *cnames;
    double time;
    int *shp, ret, i, rank, ndata, ncoords, j, *coordpos, *coordrank;
    double *coords, *data;
    FofX retData;

    strcpy(fn_sdf,filename);
    tl    = tLevel;

    if ( (ret = gft_read_rank(fn_sdf, tl, &rank))!=1 ) {
        //cout << "Error at gft_read_rank in load_file" << endl;
        retData.time = -1;
        return(retData);
    }

    shp=ivec_alloc_n(rank,"load_file:shp");
    coordpos=ivec_alloc_n(rank,"load_file:coordpos");
    coordrank=ivec_alloc_n(rank,"load_file:coordrank");

    if ( (ret = gft_read_shape(fn_sdf, tl, shp))!=1 ) {
        cout <<"Error at gft_read_shape in load_file"<<endl;
        statusBar()->showMessage("Error at gft_read_shape in load_file",3000);
        retData.time = -1;
        return(retData);
    }
    ndata=1; ncoords=0;
    for (i=0;i<rank;i++) {
        coordrank[i]=ndata;
        coordpos[i]=ncoords;
        ndata*=shp[i];
        ncoords+=shp[i];
    }

    data=vec_alloc_n(ndata,"load_file:data");
    coords=vec_alloc_n(ncoords,"load_file:coords");
    cnames=cvec_alloc_n(rank*2,"load_file:cnames");

    if ( (ret = gft_read_full(fn_sdf, tl, shp, cnames, rank, &time, coords, data))!=1 ) {
        cout <<"Error at gft_read_full in load_file"<<endl;
        statusBar()->showMessage("Error at gft_read_full in load_file",3000);
        retData.time = -1;
        return(retData);
    }

    //printf("# time= %g\n",time);
    retData.time = time;

    for (i=0;i<ndata;i++) {
        for (j=0;j<rank;j++) {
            //printf("%c= %10.5e ",cnames[j*2],coords[ coordpos[j]+(i/coordrank[j])%shp[j] ]);
            retData.X.append(coords[ coordpos[j]+(i/coordrank[j])%shp[j] ]);
        }
        //printf("value= %10.5e\n",data[i]);
        retData.Y.append(data[i]);
    }
    return retData;
    //end openSdfFile
}
void xvsMainWindow::openSdfFile()
{
    //gets file name from file dialog window
    QString fileName = QFileDialog::getOpenFileName(this,tr("Open SDF"), QDir::currentPath(), tr("SDF Files (*.sdf)"));

    if (fileName.isEmpty()) return;

    //initializes QFile object- file name
    QFile file(fileName);

    //ends if file is not readable
    if (!file.open(QIODevice::ReadOnly | QIODevice::Text)){
        statusBar()->showMessage("File unreadable. Check file format.",3000);
        return;
    }

    xytVector data;
    FofX ret;

    int i=1;
    ret = readSdfFile(qPrintable(fileName),i);
    while (ret.time != -1) {
        data.Y.append(ret.Y);
        data.X.append(ret.X);
        data.T.append(ret.time);
        i++;
        ret = readSdfFile(qPrintable(fileName),i); //reads data from single time step
        //cout << i << endl;
    }

    //create new plot window for data
    addPlotWidget();
    PlotWidget *w = arrayPlotWidgets[activeWidgetIndex];

    //initialize name label
    QStringList curDir = fileName.split('/');
    QString name;
    name = curDir.last().remove(".sdf");
    data.name = name;
    w->changeName(name);

    //set plot data
    w->setDataVec(data);

    //index label
    w->indexLabel= QString("%1/ %2").arg(0).arg(data.Y.count()-1);

    //setup maximum index value
    controlDialog->opsIndexLine->setMaximum(w->getMaxTimeIndex());
    controlDialog->viewIndexLine->setMaximum(w->getMaxTimeIndex());
    controlDialog->browseIndexLine->setMaximum(w->getMaxTimeIndex());
    controlDialog->opsIndexLine->setMinimum(0);
    controlDialog->viewIndexLine->setMinimum(0);
    controlDialog->browseIndexLine->setMinimum(0);

    //fits boundaries to data max&mins
    w->initializePlotBoundaries();
    updateControlDialogValues("updateAll");

    statusBar()->showMessage(name+".sdf opened successfully",3000);
}

/**********export routines**********/
QList<int> xvsMainWindow::getCurrentPlots()
{
    QList<int> wArray;
    if (numPlotWidgets>0) {
        if (allCheckBoxState)
        {
            for (int i=0;i<numPlotWidgets;i++) wArray.append(i);
        }
        else {
            if (selectedWidgets.count() >0){
                for (int i=0;i<selectedWidgets.count();i++)
                    wArray.append(selectedWidgets.at(i)->getIndex() );
            }
            else wArray.append(activeWidgetIndex);
        }//end else not allCheckBoxState
    }//end if plots>0
    return wArray;
}
void xvsMainWindow::exportToTextFile()
{
    QList<int> wArray = getCurrentPlots();
    //catch possible error when no plots exist
    if (numPlotWidgets<=0) {
        QMessageBox::StandardButton reply;
        reply= QMessageBox::information(this,"No data to export","Error: No plots to export");
    }
    else {
        //open save dialog
        QString filename = QFileDialog::getSaveFileName(this, tr("Save as text"),
                                                        QDir::currentPath(),
                                                        tr("Text Files (*.txt *.dat)"));
        filename.remove(".txt");
        filename.remove(".dat");
        filename.append(".txt");

        QFile file(filename);
        if (!file.open(QIODevice::WriteOnly | QIODevice::Text)) {
            statusBar()->showMessage("Could not write to file. Sorry");
            return;
        }

        QTextStream out(&file);
        out.setRealNumberNotation(QTextStream::ScientificNotation);
        out.setRealNumberPrecision(5);

        //begin writing to file
        out << "NumDatasets: " << wArray.count() << "\n";
        for (int j=0;j<wArray.count();j++) {
            PlotWidget *w = arrayPlotWidgets[wArray.at(j)];
            out << "\n";
            out << w->data.name << "\n";
            out << w->data.T.count() << "\n";
            out << "coords";
            for (int k=0;k < w->data.X.at(0).count();k++) out << "\t"<< w->data.X.at(0).at(k);
            out << "\n";
            for (int l=0;l < w->data.T.count();l++){
                out << w->data.T.at(l);
                for (int i=0;i < w->data.X.at(0).count(); i++) out << "\t"<< w->data.Y.at(l).at(i);
                out << "\n";
            }//end for loop -> times and Yvalues
        }
        out.flush();
        statusBar()->showMessage(QString("%1 datasets exported to: "+filename).arg(wArray.count()) );
    }//end else
}
void xvsMainWindow::exportToSdfFile()
{
    if (numPlotWidgets<=0) {
        QMessageBox::StandardButton reply;
        reply= QMessageBox::information(this,"No data to export","Error: No plots to export");
        statusBar()->showMessage("Export to SDF failed: No plot windows to export.",3000);
        return;
    }

    int ret, nindex_out, rank=1;
    int *shape,ndata;
    double *coords, *data;
    char *cnames,fname[100];

    QString filename = QFileDialog::getSaveFileName(this, tr("Export to SDF"),
                                                    QDir::currentPath(),
                                                    tr("SDF (*.sdf)"));
    filename.remove(".sdf");
    filename.append(".sdf");
    strcpy(fname,qPrintable(filename));

    xytVector vec = arrayPlotWidgets[activeWidgetIndex]->data;

    nindex_out = vec.T.length();
    if( nindex_out <1 ){
        cout<< "No data to export"<<endl;
        statusBar()->showMessage("No data to export",2000);
        return;
    }
    shape=ivec_alloc_n(rank,"load_file:shp");

    data=vec_alloc_n(ndata,"load_file:data");
    coords=vec_alloc_n(ndata,"load_file:coords");
    cnames=cvec_alloc_n(2,"load_file:cnames");

    /*----------------------------------------------------------------------
    c        Write the .sdf file using the RNPL/BBHTUIL interface
    c---------------------------------------------------------------------*/
    cnames[0] = 'x';

    //time index iteration
    for (int it = 0; it<nindex_out;it++) {

        shape[0] = vec.X.at(it).length();

        //update y-values
        for (int j=0;j<shape[0];j++) {
            data[j] = vec.Y.at(it).at(j);
            coords[j] = vec.X.at(it).at(j);
        }
        //write time step to file
        ret = gft_out_full(fname,vec.T.at(it),shape,cnames,1,coords,data) ;
    }//end for

    gft_close_all();
}

//stub for calling plotting routine
void xvsMainWindow::callPlottingRoutine()
{
    PlotWidget *w = arrayPlotWidgets[activeWidgetIndex];
    int tl = w->getTimeIndex();
    //int time = w->data.T.at(tl);
    int n =w->data.X.at(tl).count();

    double xValues[n];
    double yValues[n];    

    for (int i=0;i<n;i++) {
        xValues[i] = w->data.X.at(tl).at(i);
        yValues[i] = w->data.Y.at(tl).at(i);
    }


    QString filename = QDir::currentPath() +w->getName()+tl+".xy";

    QFile file(filename);
    if (!file.open(QIODevice::WriteOnly | QIODevice::Text)) {
        statusBar()->showMessage("Could not write to file. Sorry");
        return;
    }
    QTextStream out(&file);
    out.setRealNumberNotation(QTextStream::ScientificNotation);
    out.setRealNumberPrecision(6);

    //begin writing to file
    out << "x   \t y\n";
    for (int i=0;i<n;i++) {
        out << xValues[i] << "\t"<< yValues[i]<<"\n";
    }
    out.flush();

    //exit file
    file.close();

    statusBar()->showMessage("exported to xy format");
}
/********end export routines********/

/************************End File  section*************************/



/************************Applicator Functions**********************/

//apply xy menu
void xvsMainWindow::x_log_c_plus_x()
{
    double c;
    bool ok;
    c = QInputDialog::getDouble(this, "x->log(c+x)","Enter const",1,-2147483647,2147483647,3,&ok);
    if (ok) {
        applyHistMenu->addAction(x_log_c_plus_xAction);
        if (!controlButtonDepressed) cloneActivePlot();
        arrayPlotWidgets[activeWidgetIndex]->data.x_log_c_plus_x(c);
        arrayPlotWidgets[activeWidgetIndex]->refreshPixmap();
        statusBar()->showMessage("x->log(c+x)",2000);
    }
}
void xvsMainWindow::x_log_one_plus_x()
{
    applyHistMenu->addAction(x_log_one_plus_xAction);
    if (!controlButtonDepressed) cloneActivePlot();
    arrayPlotWidgets[activeWidgetIndex]->data.x_log_c_plus_x(1);
    arrayPlotWidgets[activeWidgetIndex]->refreshPixmap();
    statusBar()->showMessage("x->log(1+x)",2000);
}
void xvsMainWindow::x_times_y()
{
    applyHistMenu->addAction(x_times_yAction);
    if (!controlButtonDepressed) cloneActivePlot();
    arrayPlotWidgets[activeWidgetIndex]->data.x_times_y();
    arrayPlotWidgets[activeWidgetIndex]->refreshPixmap();
    statusBar()->showMessage("x*y",2000);
}
void xvsMainWindow::x_p()
{
    double p;
    bool ok;
    p = QInputDialog::getDouble(this, "x^p","Enter const",1,-2147483647,2147483647,3,&ok);
    if (ok) {
        applyHistMenu->addAction(x_pAction);
        if (!controlButtonDepressed) cloneActivePlot();
        arrayPlotWidgets[activeWidgetIndex]->data.x_p(p);
        arrayPlotWidgets[activeWidgetIndex]->refreshPixmap();
        statusBar()->showMessage("x^p",2000);
    }
}
void xvsMainWindow::x_p_times_y()
{
    double p;
    bool ok;
    p = QInputDialog::getDouble(this, "x^p * y","Enter const",1,-2147483647,2147483647,3,&ok);
    if (ok) {
        applyHistMenu->addAction(x_p_times_yAction);
        if (!controlButtonDepressed) cloneActivePlot();
        arrayPlotWidgets[activeWidgetIndex]->data.x_p_times_y(p);
        arrayPlotWidgets[activeWidgetIndex]->refreshPixmap();
        statusBar()->showMessage("x^p * y",2000);
    }
}
void xvsMainWindow::y_p()
{
    double p;
    bool ok;
    p = QInputDialog::getDouble(this, "y^p","Enter const",1,-2147483647,2147483647,3,&ok);
    if (ok) {
        applyHistMenu->addAction(y_pAction);
        if (!controlButtonDepressed) cloneActivePlot();
        arrayPlotWidgets[activeWidgetIndex]->data.x_p(p);
        arrayPlotWidgets[activeWidgetIndex]->refreshPixmap();
        statusBar()->showMessage("y^p",2000);
    }
}
void xvsMainWindow::y_divide_x()
{
    applyHistMenu->addAction(y_divide_xAction);
    if (!controlButtonDepressed) cloneActivePlot();
    arrayPlotWidgets[activeWidgetIndex]->data.y_divide_x();
    arrayPlotWidgets[activeWidgetIndex]->refreshPixmap();
    statusBar()->showMessage("y/x",2000);
}
void xvsMainWindow::dy_dx()
{
    applyHistMenu->addAction(dy_dxAction);
    if (!controlButtonDepressed) cloneActivePlot();
    PlotWidget *w =arrayPlotWidgets[activeWidgetIndex];
    w->changeName(w->getName()+"'");
    w->data.dy_dx();
    w->initializePlotBoundaries();
    w->refreshPixmap();
    statusBar()->showMessage("dy/dx",2000);
}

//apply n menu
bool xvsMainWindow::levelFormatted(QList<PlotWidget *>list)
{
    bool ok=true;
    int xLength = list.at(0)->data.X.at(0).count();
    int tLength = list.at(0)->data.T.count();

    for (int i =1;i<list.count();i++) {
        for (int t=0;t<list.at(i)->data.X.count();t++) {//checks for non-uniform grid
            if (list.at(i)->data.X.at(t).count() != list.at(i)->data.X.at(0).count()) {
                ok = false;
                cout << "Error in level format(xLengths). Non-uniform grid"<<endl;
                break;
            }//end if
        }
        xLength *=2;
        if (xLength != list.at(i)->data.X.at(0).count()) {
            ok = false;
            cout << "Error in level format(xLengths). expected"<<xLength<<" actual"<< list.at(i)->data.X.at(0).count()<<endl;
            break;
        }//end if
        tLength *=2;
        if (tLength != list.at(i)->data.T.count()) {
            ok = false;
            cout << "Error in level format(tLengths). expected"<<tLength<<" actual"<<list.at(i)->data.T.count()<<endl;
            break;
        }//end if
    }//end for loop
    return ok;
}
QList<xytVector> xvsMainWindow::getDiffs(QList<PlotWidget*> wgs, double p)
{
    int numWidgets = wgs.count();
    QList<xytVector> listDiffs;
    int xLength;
    int tLength = wgs.at(0)->data.T.count();
    for (int w=numWidgets-1;w>0;w--)
    {
        xytVector yDiffs;
        yDiffs.name = QString("level_%1_%2").arg(w).arg(w-1);
        for (int t=0;t<tLength;t++)
        {
            QList<double> y_at_t;
            xLength = wgs.at(w-1)->data.X.at(0).count();
            for (int x=0;x<xLength;x++)
            {
                double diff= wgs.at(w)->data.Y.at(pow(2.,w)*t).at(2*x) -
                             wgs.at(w-1)->data.Y.at(pow(2.,w-1)*t).at(x);
                y_at_t.append(diff * (pow(p,(w-1)) ) );
            }//end x loop
            yDiffs.Y.append(y_at_t);
        }//end time loop
        yDiffs.X = wgs.at(w-1)->data.X;
        yDiffs.T = wgs.at(0)->data.T;
        listDiffs.append(yDiffs);
    }//end outer widget loop
    return listDiffs;
}
void xvsMainWindow::h_sq_convergence_test()
{
    pt2Member = &xvsMainWindow::h_sq_convergence_test; //function pointer saved while selection of widgets conducted.
    if (selectedWidgets.count() > 2) {

        if (!levelFormatted(selectedWidgets)) {
            QMessageBox::information(this,"Convergence Test",
                                     "Datasets do not have the length*(2^index) scaling necessary for convergence testing");
            deactivateSelectionMode();
        }
        else{
            //subtraction of datasets
            QList<xytVector> diffs;
            diffs = getDiffs(selectedWidgets, 4.);

            //setup plot
            addPlotWidget();
            PlotWidget *w = arrayPlotWidgets[activeWidgetIndex];
            w->curveMap = diffs;
            w->data = diffs[0];
            w->changeName(selectedWidgets.at(0)->getName()+"_convTest");

            statusBar()->showMessage("O(h^2) convergence test " +selectedWidgets.at(0)->getName(),3000);

            deactivateSelectionMode();
            applyHistMenu->addAction(h_sq_convergence_testAction);
        }//end else
    }//end if
    else activateSelectionMode(-1); //will allow selection of 4 widgets
}
void xvsMainWindow::h_lin_convergence_test()
{
    pt2Member = &xvsMainWindow::h_lin_convergence_test; //function pointer saved while selection of widgets conducted.
    if (selectedWidgets.count() > 2) {

        if (!levelFormatted(selectedWidgets)) {
            QMessageBox::information(this,"Convergence Test",
                                     "Datasets do not have the length*(2^index) scaling necessary for convergence testing");
            deactivateSelectionMode();
        }
        else{
            //subtraction of datasets
            QList<xytVector> diffs;
            diffs = getDiffs(selectedWidgets, 4.);

            //setup plot
            addPlotWidget();
            PlotWidget *w = arrayPlotWidgets[activeWidgetIndex];
            w->curveMap = diffs;
            w->data = diffs[0];
            w->changeName(selectedWidgets.at(0)->getName()+"_convTest");

            statusBar()->showMessage("O(h) convergence test " +selectedWidgets.at(0)->getName(),3000);

            deactivateSelectionMode();
            applyHistMenu->addAction(h_sq_convergence_testAction);
        }//end else
    }//end if
    else activateSelectionMode(-1); //will allow selection of 4 widgets
}
void xvsMainWindow::h_p_convergence_test()
{
    pt2Member = &xvsMainWindow::h_p_convergence_test; //function pointer saved while selection of widgets conducted.
    if (selectedWidgets.count() > 2) {

        if (!levelFormatted(selectedWidgets)) {
            QMessageBox::information(this,"Convergence Test",
                                     "Datasets do not have the length*(2^index) scaling necessary for convergence testing");
            deactivateSelectionMode();
        }
        else{
            bool ok;
            double p = QInputDialog::getDouble(this, "O(h^p)","Enter const",1,-2,10,1,&ok);
            if (ok) {

                //subtraction of datasets
                QList<xytVector> diffs;
                diffs = getDiffs(selectedWidgets, pow(2,p) );

                //setup plot
                addPlotWidget();
                PlotWidget *w = arrayPlotWidgets[activeWidgetIndex];
                w->curveMap = diffs;
                w->data = diffs[0];
                w->changeName(selectedWidgets.at(0)->getName()+"_convTest");

                statusBar()->showMessage("O(h^p) convergence test " +selectedWidgets.at(0)->getName(),3000);

                deactivateSelectionMode();
                applyHistMenu->addAction(h_sq_convergence_testAction);
            }
        }//end else
    }//end if
    else activateSelectionMode(-1); //will allow selection of arbitrary number of widgets
}

//compare menu
bool xvsMainWindow::compare_x(PlotWidget *w1, PlotWidget *w2, double fuzz)
{
    //check for time step mismatches
    if (w1->data.X.count() != w2->data.X.count()) {
        QMessageBox::StandardButton reply;
        reply= QMessageBox::information(this,"Compare Full Datasets","Error: X data have different number of time steps");
        return false;
    }

    for (int i=0;i<w1->data.X.count();i++){ //loop over time
        for (int j=0;j<w1->data.X.at(i).count();j++){ //loop over x coords

            //check for non-uniform grid mismatches
            if (w1->data.X.at(i).count() != w2->data.X.at(i).count()) {
                QMessageBox::StandardButton reply;
                reply= QMessageBox::information(this,"Compare Full Datasets","Error: X data have different lengths");
                return false;
            }

            //compare values
            if (fabs(w1->data.X.at(i).at(j) - w2->data.X.at(i).at(j)) > fuzz) return false;

        }//end x-loop
    }//end time loop
    return true;
}
bool xvsMainWindow::compare_y(PlotWidget *w1, PlotWidget *w2, double fuzz)
{
    if (w1->data.Y.count() != w2->data.Y.count()) {
        QMessageBox::StandardButton reply;
        reply= QMessageBox::information(this,"Compare Full Datasets","Error: Y data different number of time steps");
        return false;
    }
    for (int i=0;i<w1->data.Y.count();i++){
        for(int j=0;j<w1->data.Y.at(i).count();j++){

            //check for non-uniform grid mismatches
            if (w1->data.Y.at(i).count() != w2->data.Y.at(i).count()) {
                QMessageBox::StandardButton reply;
                reply= QMessageBox::information(this,"Compare Full Datasets","Error: Y data have different lengths");
                return false;
            }

            //compare values
            if (fabs(w1->data.Y.at(i).at(j) - w2->data.Y.at(i).at(j)) >fuzz) return false;
        }
    }
    return true;
}
bool xvsMainWindow::compare_t(PlotWidget *w1, PlotWidget *w2, double fuzz)
{
    if (w1->data.T.count() != w2->data.T.count()) {
        QMessageBox::StandardButton reply;
        reply= QMessageBox::information(this,"Compare Full Datasets","Error: T data different lengths");
    }
    for (int i=0;i<w1->data.T.count();i++){
        if (fabs(w1->data.T.at(i) - w2->data.T.at(i)) > fuzz) return false;
    }
    return true;
}

void xvsMainWindow::compare_full_exact()
{
    bool equal = true;

    pt2Member = &xvsMainWindow::compare_full_exact; //function pointer saved while selection of widgets conducted.
    if (selectedWidgets.count() == 2) {
        PlotWidget *w1 = selectedWidgets[0];
        PlotWidget *w2 = selectedWidgets[1];

        //comparison of datasets
        equal = compare_x(w1,w2,0.0) && compare_y(w1,w2,0.0) && compare_t(w1,w2,0.0);
        QMessageBox::StandardButton reply;
        QString answer;

        //popup dialog
        if (equal) answer = QString("Datasets X,Y,T equal to double precision");
        else answer = QString("Datasets X,Y,T not equal to double precision");
        reply= QMessageBox::information(this,"Compare Full Datasets",answer);

        deactivateSelectionMode();
        applyHistMenu->addAction(compare_full_exactAction);
    }
    else activateSelectionMode(2); //will allow selection of 2 widgets
}
void xvsMainWindow::compare_full_fuzz()
{
    bool equal = true;

    pt2Member = &xvsMainWindow::compare_full_fuzz; //function pointer saved while selection of widgets conducted.
    if (selectedWidgets.count() == 2) {
        PlotWidget *w1 = selectedWidgets[0];
        PlotWidget *w2 = selectedWidgets[1];

        equal = compare_x(w1,w2,ifuzz) && compare_y(w1,w2,ifuzz) && compare_t(w1,w2,ifuzz);
        QMessageBox::StandardButton reply;
        QString answer;
        if (equal) answer = QString("Datasets X,Y,T equal to fuzz precision");
        else answer = QString("Datasets X,Y,T not equal to fuzz precision");
        reply= QMessageBox::information(this,"Compare Full Datasets",answer);

        deactivateSelectionMode();
        applyHistMenu->addAction(compare_full_fuzzAction);
    }
    else activateSelectionMode(2);
}
void xvsMainWindow::compare_x_exact()
{
    bool equal = true;

    pt2Member = &xvsMainWindow::compare_x_exact; //function pointer saved while selection of widgets conducted.
    if (selectedWidgets.count() == 2) {
        PlotWidget *w1 = selectedWidgets[0];
        PlotWidget *w2 = selectedWidgets[1];

        equal = compare_x(w1,w2,0.0);
        QMessageBox::StandardButton reply;
        QString answer;
        if (equal) answer = QString("X dataset equal to double precision");
        else answer = QString("X dataset not equal to double precision");
        reply= QMessageBox::information(this,"Compare x Datasets",answer);

        deactivateSelectionMode();
        applyHistMenu->addAction(compare_x_exactAction);
    }
    else activateSelectionMode(2);
}
void xvsMainWindow::compare_x_fuzz()
{
    bool equal = true;

    pt2Member = &xvsMainWindow::compare_x_fuzz; //function pointer saved while selection of widgets conducted.
    if (selectedWidgets.count() == 2) {
        PlotWidget *w1 = selectedWidgets[0];
        PlotWidget *w2 = selectedWidgets[1];

        equal = compare_x(w1,w2,ifuzz);
        QMessageBox::StandardButton reply;
        QString answer;
        if (equal) answer = QString("X dataset equal to fuzz precision");
        else answer = QString("X dataset not equal to fuzz precision");
        reply= QMessageBox::information(this,"Compare x Datasets",answer);

        deactivateSelectionMode();
        applyHistMenu->addAction(compare_x_fuzzAction);
    }
    else activateSelectionMode(2);
}
void xvsMainWindow::compare_y_exact()
{
    bool equal = true;

    pt2Member = &xvsMainWindow::compare_y_exact; //function pointer saved while selection of widgets conducted.
    if (selectedWidgets.count() == 2) {
        PlotWidget *w1 = selectedWidgets[0];
        PlotWidget *w2 = selectedWidgets[1];

        equal = compare_y(w1,w2,0.0);
        QMessageBox::StandardButton reply;
        QString answer;
        if (equal) answer = QString("Y dataset equal to double precision");
        else answer = QString("Y dataset not equal to double precision");
        reply= QMessageBox::information(this,"Compare y Datasets",answer);

        deactivateSelectionMode();
        applyHistMenu->addAction(compare_y_exactAction);
    }
    else activateSelectionMode(2);
}
void xvsMainWindow::compare_y_fuzz()
{
    bool equal = true;

    pt2Member = &xvsMainWindow::compare_y_fuzz; //function pointer saved while selection of widgets conducted.
    if (selectedWidgets.count() == 2) {
        PlotWidget *w1 = selectedWidgets[0];
        PlotWidget *w2 = selectedWidgets[1];

        equal = compare_y(w1,w2,ifuzz);
        QMessageBox::StandardButton reply;
        QString answer;
        if (equal) answer = QString("Y dataset equal to fuzz precision");
        else answer = QString("Y dataset not equal to fuzz precision");
        reply= QMessageBox::information(this,"Compare y Datasets",answer);

        deactivateSelectionMode();
        applyHistMenu->addAction(compare_y_fuzzAction);
    }
    else activateSelectionMode(2);
}
void xvsMainWindow::compare_t_exact()
{
    bool equal = true;

    pt2Member = &xvsMainWindow::compare_t_exact; //function pointer saved while selection of widgets conducted.
    if (selectedWidgets.count() == 2) {
        PlotWidget *w1 = selectedWidgets[0];
        PlotWidget *w2 = selectedWidgets[1];

        equal = compare_t(w1,w2,0.0);
        QMessageBox::StandardButton reply;
        QString answer;
        if (equal) answer = QString("T datasets equal to double precision");
        else answer = QString("T datasets not equal to double precision");
        reply= QMessageBox::information(this,"Compare t Datasets",answer);

        deactivateSelectionMode();
        applyHistMenu->addAction(compare_t_exactAction);
    }
    else activateSelectionMode(2);
}
void xvsMainWindow::compare_t_fuzz()
{
    bool equal = true;

    pt2Member = &xvsMainWindow::compare_t_fuzz; //function pointer saved while selection of widgets conducted.
    if (selectedWidgets.count() == 2) {
        PlotWidget *w1 = selectedWidgets[0];
        PlotWidget *w2 = selectedWidgets[1];

        equal = compare_t(w1,w2,ifuzz);
        QMessageBox::StandardButton reply;
        QString answer;
        if (equal) answer = QString("T datasets equal to fuzz precision");
        else answer = QString("T datasets not equal to fuzz precision");
        reply= QMessageBox::information(this,"Compare t Datasets",answer);

        deactivateSelectionMode();
        applyHistMenu->addAction(compare_t_fuzzAction);
    }
    else activateSelectionMode(2);
}
void xvsMainWindow::compare_xy_exact()
{
    bool equal = true;

    pt2Member = &xvsMainWindow::compare_xy_exact; //function pointer saved while selection of widgets conducted.
    if (selectedWidgets.count() == 2) {
        PlotWidget *w1 = selectedWidgets[0];
        PlotWidget *w2 = selectedWidgets[1];

        equal = compare_x(w1,w2,0.0) && compare_y(w1,w2,0.0);
        QMessageBox::StandardButton reply;
        QString answer;
        if (equal) answer = QString("Datasets X,Y equal to double precision");
        else answer = QString("Datasets X,Y not equal to double precision");
        reply= QMessageBox::information(this,"Compare xy Datasets",answer);

        deactivateSelectionMode();
        applyHistMenu->addAction(compare_xy_exactAction);
    }
    else activateSelectionMode(2);
}
void xvsMainWindow::compare_xy_fuzz()
{
    bool equal = true;

    pt2Member = &xvsMainWindow::compare_xy_fuzz; //function pointer saved while selection of widgets conducted.
    if (selectedWidgets.count() == 2) {
        PlotWidget *w1 = selectedWidgets[0];
        PlotWidget *w2 = selectedWidgets[1];

        equal = compare_x(w1,w2,ifuzz) && compare_y(w1,w2,ifuzz);
        QMessageBox::StandardButton reply;
        QString answer;
        if (equal) answer = QString("Datasets X,Y equal to fuzz precision");
        else answer = QString("Datasets X,Y not equal to fuzz precision");
        reply= QMessageBox::information(this,"Compare xy Datasets",answer);

        deactivateSelectionMode();
        applyHistMenu->addAction(compare_xy_fuzzAction);
    }
    else activateSelectionMode(2);
}

//merge&split
bool xvsMainWindow::isMergable(QList<PlotWidget*> plots)
{
    bool ret = true;
    for (int i=0;i<plots.length();i++) {
        if (plots.at(0)->data.T.length() != plots.at(i)->data.T.length() ) {
            ret = false;
            break;
        }//end if
    }//end for
    return ret;
}
void xvsMainWindow::merge()
{
    pt2Member = &xvsMainWindow::merge; //function pointer saved while selection of widgets conducted.
    if (selectedWidgets.count() == 2) {
        PlotWidget *w1 = selectedWidgets[0];
        PlotWidget *w2 = selectedWidgets[1];

        //comparison of datasets
        QString answer;
        if (isMergable(selectedWidgets)) {
            applyHistMenu->addAction(mergeAction);

            //logic will create merged plot in 1st window if control
            //button is depressed and remove the 2nd window. otherwise
            //new window with both datasets will be created
            PlotWidget *w;
            if (!controlButtonDepressed) {
                cloneActivePlot();
                w =arrayPlotWidgets[activeWidgetIndex];
                w->setDataVec(w1->data);
            }
            else w =w1;

            //merge data
            w->addCurveMap(w1->data);
            w->addCurveMap(w2->data);
            w->changeName(w1->getName()+" & "+w2->getName());
            w->data.name = w1->getName();
            w->initializePlotBoundaries();
            w->refreshPixmap();

            if (controlButtonDepressed) {
                changeActivePlot(w2->getIndex());
                removePlotWidget();
                changeActivePlot(w1->getIndex());
            }
            answer = QString("Datasets merged");
            statusBar()->showMessage(answer,2000);
        }
        else {
            //popup dialog
            answer = QString("Datasets could not be merged. #Time steps unequal");
            QMessageBox::StandardButton reply;
            reply= QMessageBox::information(this,"Datasets",answer);
        }
        deactivateSelectionMode();
    }
    else activateSelectionMode(2); //will allow selection of 2 widgets
}
void xvsMainWindow::split()
{
    pt2Member = &xvsMainWindow::split; //function pointer saved while selection of widgets conducted.
    if (selectedWidgets.count() == 1) {
        PlotWidget *w = selectedWidgets[0];

        //comparison of datasets
        QString answer;
        if (w->curveMap.length() >1) {
            applyHistMenu->addAction(splitAction);

            //will create plots for each function in original window
            PlotWidget *n;
            for (int i=0;i<w->curveMap.length();i++) {
                addPlotWidget();
                n = arrayPlotWidgets[activeWidgetIndex];
                n->setDataVec(w->curveMap.at(i));

                //set plotsettings
                n->setPlotBoundaries(w->getPlotLimits());

                //set parameters
                n->setTimeIndex(w->getTimeIndex());
                n->changeName(n->data.name);
            }

            //remove original plot
            if (controlButtonDepressed) {
                changeActivePlot(w->getIndex());
                removePlotWidget();
            }

            answer = QString("Datasets split");
            statusBar()->showMessage(answer,2000);
        }
        else {
            //popup dialog
            answer = QString("Error: single dataset cannot not be split.");
            QMessageBox::StandardButton reply;
            reply= QMessageBox::information(this,"Datasets",answer);
        }
        deactivateSelectionMode();
    }
    else activateSelectionMode(1); //will allow selection of 2 widgets
}
/**********************END Applicator Functions********************/


/*********************selectionMode********************************/
//creates separate thread to track widget selection
void xvsMainWindow::activateSelectionMode(int num)
{
    selectionMode = true; //mainWindow member function
    emit selectionModeActivated(SELECTION);//sends all widgets to selection mode
                                            //should be called changeModeTo(SELECTION)
    getSelectedWidgets *myThread;//new thread to track widget selection
    myThread = new getSelectedWidgets(num);
    connect(this,SIGNAL(selectedWidgetsChanged(int)),myThread,SLOT(changeSelected(int)));
    connect(this,SIGNAL(endSelectionMode(int)),myThread,SLOT(changeSelected(int)) );
    myThread->start(QThread::IdlePriority);
    connect(myThread,SIGNAL(finished()),this,SLOT(selectFunction()) );
    statusBar()->showMessage("Selection Mode On");
}

//resets all windows after selections finished
void xvsMainWindow::deactivateSelectionMode()
{
    emit selectionModeActivated(DESELECTION);//sends all widgets back to Zoom mode
    selectionMode =false;
    pt2Member = NULL; //removes function pointer
    while (!selectedWidgets.isEmpty()) selectedWidgets.removeFirst(); //removes widgets from selection array
    statusBar()->showMessage("Selection Mode Off",2000);
}

//adds widget to selection array & emits signal
//to be received by getSelectedWidgets thread
void xvsMainWindow::addSelectedWidget(int i)
{
    selectedWidgets.append(arrayPlotWidgets[i]);
    statusBar()->showMessage(QString("Plot #%1 selected").arg(i),1000);
    emit selectedWidgetsChanged(selectedWidgets.count());
}

//removes widget from selection array& emits signal
//to be received by getSelectedWidgets thread
void xvsMainWindow::removeSelectedWidget(int i)
{
    for(int j=0;j<selectedWidgets.count();j++) {
        if (selectedWidgets.at(j)->getIndex() == i) {
            selectedWidgets.removeAt(j);
            statusBar()->showMessage(QString("Plot #%1 deselected").arg(i),1000);
        }
    }
    emit selectedWidgetsChanged(selectedWidgets.count());
}

//calls function set by pointer
void xvsMainWindow::selectFunction()
{
    (this->*pt2Member)();
}
/*********************END selectionMode****************************/


/**********************PLOT WIDGETS SECTION**********************/

//keyboard shortcuts implemented
void xvsMainWindow::createPlotActions()
{
    killActiveWindow = new QAction(this);
    killActiveWindow->setShortcut(Qt::CTRL+Qt::Key_Space);
    connect(killActiveWindow,SIGNAL(triggered()),this,SLOT(removePlotWidget()));
    this->addAction(killActiveWindow);

    maximizeActiveWindowAction = new QAction(tr("Maximize active window"),this);
    maximizeActiveWindowAction->setShortcut(Qt::Key_Space);
    connect(maximizeActiveWindowAction,SIGNAL(triggered()),this,SLOT(maximizeActivePlot()));
    this->addAction(maximizeActiveWindowAction);
}
void xvsMainWindow::keyPressEvent(QKeyEvent *event)
{
    if (numPlotWidgets >0) {
        PlotWidget *w = arrayPlotWidgets[activeWidgetIndex];
        switch (event->key())
        {
        case Qt::Key_C: //clone window
            cloneActivePlot();
            break;
        case Qt::Key_Q: //kill window
            removePlotWidget();
            break;
        case Qt::Key_H: //get help
            getHelp();
        case Qt::Key_B: //previous window
            if (numPlotWidgets > 0 && activeWidgetIndex != 0){
                changeActivePlot(activeWidgetIndex -1);
            }
            break;
        case Qt::Key_N: //next window
            if (numPlotWidgets > 0 && activeWidgetIndex != numPlotWidgets-1){
                changeActivePlot(activeWidgetIndex +1);
            }
            break;
        case Qt::Key_M: //markers subpanel
            getControlDialog();
            controlDialog->tabWidget->setCurrentIndex(1);
            break;
        case Qt::Key_D: //data subpanel
            getControlDialog();
            controlDialog->tabWidget->setCurrentIndex(2);
            break;
        case Qt::Key_V: //view subpanel
            getControlDialog();
            controlDialog->tabWidget->setCurrentIndex(0);
            break;
        case Qt::Key_A:
            w->changeDisplayMode(AUTO); //animate mode
            break;
        case Qt::Key_Z:
            w->changeDisplayMode(XYZOOM); //zoom mode
            break;
        case Qt::Key_X:
            w->changeDisplayMode(XZOOM); //xzoom mode
            break;
        case Qt::Key_Y:
            w->changeDisplayMode(YZOOM); //yzoom mode
            break;
        case Qt::Key_E:
            w->changeDisplayMode(ZOOM); //easy zoom mode
            break;
        case Qt::Key_S:
            w->changeDisplayMode(STEP); //step mode
            break;
        case Qt::Key_F:
            w->changeDisplayMode(FREEZE); //freeze mode
            break;
        case Qt::Key_Control:
            controlButtonDepressed = true;
            break;
        case Qt::Key_W:
            int i;
            i= (w->getFullScale()+1)%2; //interchanges 0 for 1
            w->toggleFullScale(i);
            controlDialog->fullScaleComboBox->setCurrentIndex(i);
            break;
        default:
            QWidget::keyPressEvent(event);
        }//end switch
    }//end if
}
void xvsMainWindow::keyReleaseEvent(QKeyEvent *event)
{
    switch (event->key())
    {
    case Qt::Key_Control:
        controlButtonDepressed = false;
    default:
        QWidget::keyPressEvent(event);
    }
}
void xvsMainWindow::mousePressEvent(QMouseEvent *event)
{
    if (event->button() == Qt::MidButton) {
        emit endSelectionMode(-1);
        //cout << "mid button clicked" <<endl;
    }
    else QWidget::mousePressEvent(event);
}

//adds a plot widget
void xvsMainWindow::addPlotWidget()
{
    if (numPlotWidgets < 25)
    {
        if (isPlotMaximized) maximizeActivePlot(); //should rename maximizeActivePlot to toggleMaximizeActivePlot
        PlotWidget *newPWidget = new PlotWidget(this,numPlotWidgets,allCheckBoxState);
        arrayPlotWidgets[numPlotWidgets] = newPWidget;
        connect(arrayPlotWidgets[numPlotWidgets],SIGNAL(mouseMoved(QPointF,int)),
                this,SLOT(cursorPosUpdate(QPointF,int)));

        numPlotWidgets+=1;

        if (numPlotWidgets<=4) gridSize=2;
        else if ((numPlotWidgets<= 9)&&(numPlotWidgets> 4)) gridSize=3;
        else if ((numPlotWidgets<=16)&&(numPlotWidgets> 9)) gridSize=4;
        else if ((numPlotWidgets<=25)&&(numPlotWidgets>16)) gridSize=5;

        if ((numPlotWidgets==5)||(numPlotWidgets==10)||(numPlotWidgets==17)) organizePlotWidgets();
        else{
            int row = (numPlotWidgets-1)/gridSize;
            int col = (numPlotWidgets-1)%gridSize;
            gridLayout->addWidget(arrayPlotWidgets[numPlotWidgets-1],row,col);
        }
        coordLabel->clear();

        changeActivePlot(numPlotWidgets-1);

        //creates connections to control dialog
        setupPlotWindow(arrayPlotWidgets[numPlotWidgets-1]);
    }
    else{
        QMessageBox msgBox(QMessageBox::Warning, tr("Error"),
                           "Error: Could not add window. Max number of windows is 25", 0, this);
        msgBox.exec();
    }
}

//clones active plot data to new window (Note: not all display variables will be cloned)
void xvsMainWindow::cloneActivePlot()
{
    PlotWidget *oldWidget = arrayPlotWidgets[activeWidgetIndex];
    addPlotWidget();
    PlotWidget *newWidget = arrayPlotWidgets[activeWidgetIndex];

    //set data
    newWidget->setDataVec(oldWidget->data);
    for (int i=0;i<oldWidget->curveMap.length();i++) newWidget->addCurveMap(oldWidget->curveMap.at(i));

    //set plotsettings
    newWidget->setPlotBoundaries(oldWidget->getPlotLimits());

    //set parameters
    newWidget->setTimeIndex(oldWidget->getTimeIndex());
    newWidget->changeName(oldWidget->getName());
    newWidget->changeLineColor(oldWidget->getLineColor());
    newWidget->changeLineWidth(oldWidget->getLineWidth());
    newWidget->changeMarkerColor(oldWidget->getMarkerColor());
    newWidget->changeMarkerSize(oldWidget->getMarkerSize());
    newWidget->changeMarkerType(oldWidget->getMarkerType());
}

//removes active plot
void xvsMainWindow::removePlotWidget()
{
    if (numPlotWidgets > 0)
    {
        //returns from maximized view
        if (isPlotMaximized) maximizeActivePlot();

        //deletes active plot widget
        int i = activeWidgetIndex;
        numPlotWidgets-=1;
        delete arrayPlotWidgets[i];

        //reorders array of widget pointers
        for (int j=i;j<numPlotWidgets;j++)
        {
            arrayPlotWidgets[j] = arrayPlotWidgets[j+1];
            arrayPlotWidgets[j]->changeIndex(j);
        }
        //sets first plot active
        activeWidgetIndex =0;
        if (numPlotWidgets > 0){
            arrayPlotWidgets[0]->setActive();
            arrayPlotWidgets[0]->update();
        }

        //resets display grid & status labels
        organizePlotWidgets();
        coordLabel->clear();


    }//end if widget#>0
    else{
        QMessageBox msgBox(QMessageBox::Warning, tr("Error"),
                           "Error: No windows left to remove", 0, this);
        msgBox.exec();
    }//end else
}

//reshuffles plot widgets when new gridsize is required (slot)
void xvsMainWindow::organizePlotWidgets()
{
    QLayoutItem *child;
    while ((child=gridLayout->takeAt(0))!=0){
        delete child;
    }
    if (numPlotWidgets<=4) gridSize=2;
    else if ((numPlotWidgets<= 9)&&(numPlotWidgets> 4)) gridSize=3;
    else if ((numPlotWidgets<=16)&&(numPlotWidgets> 9)) gridSize=4;
    else if ((numPlotWidgets<=25)&&(numPlotWidgets>16)) gridSize=5;
    for (int i=0;i<numPlotWidgets;i++){
        int row = i/gridSize;
        int col = i%gridSize;
        gridLayout->addWidget(arrayPlotWidgets[i],row,col);
    }
}

//kills all plot windows and clears status and coord labels (slot)
void xvsMainWindow::killAllPlotWindows()
{
    for (int i=0;i<numPlotWidgets;i++) delete arrayPlotWidgets[i];
    numPlotWidgets=0;
    activeWidgetIndex=0;
    coordLabel->clear();
    statusBar()->showMessage("Killed all plot windows",2000);
}

//changes active plot to new one
void xvsMainWindow::changeActivePlot(int i)
{
    if (numPlotWidgets > 0){
        arrayPlotWidgets[activeWidgetIndex]->setDeactive();
        arrayPlotWidgets[activeWidgetIndex]->refreshPixmap();
        arrayPlotWidgets[i]->setActive();
        arrayPlotWidgets[i]->refreshPixmap();
    }
    activeWidgetIndex =i;
    statusLabel->setNum(activeWidgetIndex);
    emit widgetChanged("updateAll");
}

//updates the cursor position shown in the coordLabel
void xvsMainWindow::cursorPosUpdate(QPointF pos, int i)
{
    if ((i!=activeWidgetIndex)&&(!selectionMode) ) {
        changeActivePlot(i);
    }
    QString cursorPos = QString("%1, %2").arg(pos.x(),0,'E',3).arg(pos.y(),0,'E',3);
    coordLabel->setText(cursorPos);
}

//maximizes plot of interest and hides all others
void xvsMainWindow::maximizeActivePlot()
{
    isPlotMaximized = !isPlotMaximized;
    if (isPlotMaximized)
    {
        for(int i=0;i<numPlotWidgets;i++)
        {
            if(i != activeWidgetIndex) arrayPlotWidgets[i]->setHidden(true);
        }//end for
        statusBar()->showMessage(QString("Plot %1 maximized").arg(activeWidgetIndex),2000);
    }//end if
    else
    {
        for(int i=0;i<numPlotWidgets;i++)
        {
            arrayPlotWidgets[i]->setHidden(false);
        }//end for
    }//end else
}

//not finished creates actions specific to context menu (menu available at right click)
void xvsMainWindow::createContextActions()
{
    //kills current window
    killSingleWindowAction = new QAction(tr("kill current window"),this);
    connect(killSingleWindowAction, SIGNAL(triggered()), this, SLOT(removePlotWidget()) );

    //kills current window
    killAllWindowAction = new QAction(tr("kill all windows"),this);
    connect(killAllWindowAction, SIGNAL(triggered()), this, SLOT(killAllPlotWindows()) );

    toggle_apply_all_on = new QAction(tr("Toggle Apply To All"),this);
    toggle_apply_all_on->setCheckable(true);
    toggle_apply_all_on->setChecked(false);
    connect(toggle_apply_all_on, SIGNAL(toggled(bool)), this, SLOT(setAllCheckBoxState(bool)) );

    //applicator functions
    dy_dxAction = new QAction(tr("dy/dx"),this);
    dy_dxAction->setWhatsThis("Returns dy/dx = y'(x,t)");
    connect(dy_dxAction,SIGNAL(triggered()), this,SLOT(dy_dx()));
    applyMiscTable.append(dy_dxAction);

    x_log_c_plus_xAction = new QAction(tr("x -> log(c + x)"), this);
    connect(x_log_c_plus_xAction, SIGNAL(triggered()), this, SLOT(x_log_c_plus_x()) );
    applyMiscTable.append(x_log_c_plus_xAction);

    x_log_one_plus_xAction = new QAction(tr("x -> log(1 + x)"), this);
    connect(x_log_one_plus_xAction, SIGNAL(triggered()),this,SLOT(x_log_one_plus_x()));
    applyMiscTable.append(x_log_one_plus_xAction);

    x_times_yAction = new QAction(tr("x y"),this);
    connect(x_times_yAction,SIGNAL(triggered()),this,SLOT(x_times_y()));
    applyMiscTable.append(x_times_yAction);

    y_divide_xAction = new QAction(tr("y / x"),this);
    connect(y_divide_xAction,SIGNAL(triggered()), this,SLOT(y_divide_x()));
    applyMiscTable.append(y_divide_xAction);

    x_pAction = new QAction(tr("x^p"),this);
    connect(x_pAction,SIGNAL(triggered()), this,SLOT(x_p()));
    applyMiscTable.append(x_pAction);

    x_p_times_yAction = new QAction(tr("x^p y"),this);
    connect(x_p_times_yAction,SIGNAL(triggered()), this,SLOT(x_p_times_y()));
    applyMiscTable.append(x_p_times_yAction);

    y_pAction = new QAction(tr("y^p"),this);
    connect(y_pAction,SIGNAL(triggered()), this,SLOT(y_p()));
    applyMiscTable.append(y_pAction);

    //compareMenu applicators
    compare_full_exactAction = new QAction(tr("full exact"),this);
    compare_full_exactAction->setWhatsThis("compares two plots to machine precision: returns equal or not equal");
    connect(compare_full_exactAction,SIGNAL(triggered()), this,SLOT(compare_full_exact()) );
    compareTable.append(compare_full_exactAction);

    compare_full_fuzzAction= new QAction(tr("full fuzz"),this);
    compare_full_fuzzAction->setWhatsThis("compares two plots to fuzz tolerence: returns equal or not equal");
    connect(compare_full_fuzzAction,SIGNAL(triggered()), this,SLOT(compare_full_fuzz()) );
    compareTable.append(compare_full_fuzzAction);

    compare_x_exactAction= new QAction(tr("x exact"),this);
    connect(compare_x_exactAction,SIGNAL(triggered()), this,SLOT(compare_x_exact()));
    compareTable.append(compare_x_exactAction);

    compare_x_fuzzAction= new QAction(tr("x fuzz"),this);
    connect(compare_x_fuzzAction,SIGNAL(triggered()), this,SLOT(compare_x_fuzz()) );
    compareTable.append(compare_x_fuzzAction);

    compare_y_exactAction= new QAction(tr("y exact"),this);
    connect(compare_y_exactAction,SIGNAL(triggered()), this,SLOT(compare_y_exact()));
    compareTable.append(compare_y_exactAction);

    compare_y_fuzzAction= new QAction(tr("y fuzz"),this);
    connect(compare_y_fuzzAction,SIGNAL(triggered()), this,SLOT(compare_y_fuzz()));
    compareTable.append(compare_y_fuzzAction);

    compare_t_exactAction= new QAction(tr("t exact"),this);
    connect(compare_t_exactAction,SIGNAL(triggered()), this,SLOT(compare_t_exact()));
    compareTable.append(compare_t_exactAction);

    compare_t_fuzzAction= new QAction(tr("t fuzz"),this);
    connect(compare_t_fuzzAction,SIGNAL(triggered()), this,SLOT(compare_t_fuzz()));
    compareTable.append(compare_t_fuzzAction);

    compare_xy_exactAction= new QAction(tr("xy exact"),this);
    connect(compare_xy_exactAction,SIGNAL(triggered()), this,SLOT(compare_xy_exact()));
    compareTable.append(compare_xy_exactAction);

    compare_xy_fuzzAction= new QAction(tr("xy fuzz"),this);
    connect(compare_xy_fuzzAction,SIGNAL(triggered()), this,SLOT(compare_xy_fuzz()));
    compareTable.append(compare_xy_fuzzAction);

    //convergence test functions
    h_sq_convergence_testAction= new QAction(tr("O(h^2) convergence test"),this);
    h_sq_convergence_testAction->setWhatsThis("2nd order convergence test");
    connect(h_sq_convergence_testAction,SIGNAL(triggered()), this,SLOT(h_sq_convergence_test()));

    h_lin_convergence_testAction= new QAction(tr("O(h) convergence test"),this);
    h_lin_convergence_testAction->setWhatsThis("1st order convergence test");
    connect(h_lin_convergence_testAction,SIGNAL(triggered()), this,SLOT(h_lin_convergence_test()));

    h_p_convergence_testAction= new QAction(tr("O(h^p) convergence test"),this);
    h_p_convergence_testAction->setWhatsThis("pth order convergence test");
    connect(h_p_convergence_testAction,SIGNAL(triggered()), this,SLOT(h_p_convergence_test()));

    //Merge and split actions
    mergeAction= new QAction(tr("Merge"),this);
    mergeAction->setWhatsThis("Merges two widgets with equal time steps");
    connect(mergeAction,SIGNAL(triggered()),this,SLOT(merge()) );

    splitAction= new QAction(tr("Split"),this);
    splitAction->setWhatsThis("Splits single widgets with multiple functions");
    connect(splitAction,SIGNAL(triggered()),this,SLOT(split()) );
    //connect(,SIGNAL(triggered()), this,SLOT());
}

//creates menus in context menu
void xvsMainWindow::createContextMenus()
{
    apply_control_menu = new QMenu(tr("Apply to all option"), this);
    apply_control_menu->addAction(toggle_apply_all_on);

    kill_window_menu = new QMenu(tr("Kill Window"), this);
    kill_window_menu->addAction(killSingleWindowAction);
    kill_window_menu->addAction(killAllWindowAction);

    apply1Menu = new QMenu(tr("Apply XY"), this);
    apply1Menu->addActions(applyMiscTable);

    applyHistMenu = new QMenu(tr("History"),this);
    applyHistMenu->addAction(dy_dxAction);

    compareMenu = new QMenu(tr("compare"),this);
    compareMenu->addActions(compareTable);

    applyNMenu = new QMenu(tr("Convergence Tests"),this);
    applyNMenu->addAction(h_sq_convergence_testAction);
    applyNMenu->addAction(h_lin_convergence_testAction);
    applyNMenu->addAction(h_p_convergence_testAction);
}

//creates context menu on right click
void xvsMainWindow::contextMenuEvent(QContextMenuEvent *event)
{
    PlotWidget *w = arrayPlotWidgets[activeWidgetIndex];
    if ((!selectionMode) && (numPlotWidgets >0) && (XZOOM != w->getDisplayMode()) && (YZOOM != w->getDisplayMode()) && (XYZOOM != w->getDisplayMode())) {
        QMenu menu(this);
        menu.addAction(showControlDialogAction);
        menu.addMenu(applyHistMenu);
        menu.addMenu(kill_window_menu);
        menu.addMenu(apply_control_menu);
        menu.addMenu(apply1Menu);
        menu.addMenu(compareMenu);
        menu.addMenu(applyNMenu);
        menu.addAction(mergeAction);
        menu.addAction(splitAction);
        menu.exec(event->globalPos());
    }
}

//creates status bar
void xvsMainWindow::createStatusBar()
{
    statusLabel = new QLabel(" W999 ");
    statusLabel->setAlignment(Qt::AlignHCenter);
    statusLabel->setMinimumSize(statusLabel->sizeHint());
    formulaLabel = new QLabel;
    formulaLabel->setIndent(3);
    statusBar()->addWidget(statusLabel);
    statusBar()->addWidget(formulaLabel, 1);
    statusLabel->setNum(activeWidgetIndex);
}
/********************END PLOT WIDGETS SECTION********************/



/************************************************************/
/*********************MENU BAR SECTION***********************/
/************************************************************/
//creates menu actions & connects to slots
void xvsMainWindow::createMenuActions()
{
    //menu item- sets insert fuzz (generates lineinput)
    set_insert_fuzz = new QAction(tr("Set insert fuzz"),this);
    connect(set_insert_fuzz, SIGNAL(triggered()), this, SLOT(getInsertFuzz()) );

    //resets insert fuzz (?)
    reset_insert_fuzz = new QAction(tr("Reset insert fuzz"),this);
    connect(reset_insert_fuzz, SIGNAL(triggered()), this, SLOT(ResetInsertFuzz()) );

    //sets up accelerator keys for closing windows
    ctrl_space_kills_windows = new QAction(tr("Ctrl-space kills windows"),this);
    ctrl_space_kills_windows->setCheckable(true);
    ctrl_space_kills_windows->setChecked(true);
    connect(ctrl_space_kills_windows,SIGNAL(toggled(bool)),
            this,SLOT(toggle_ctrl_space_kills_windows(bool)));

    //exit
    exitHandlerAction = new QAction(tr("XVS Exit"),this);
    connect(exitHandlerAction,SIGNAL(triggered()),this,SLOT(exitHandeler()) );

    //open control dialog
    showControlDialogAction = new QAction(tr("Show Control Dialog"),this);
    connect(showControlDialogAction,SIGNAL(triggered()),this,SLOT(getControlDialog()));

    //help link
    getHelpAction = new QAction(tr("Help/Doc"),this);
    connect(getHelpAction,SIGNAL(triggered()),this,SLOT(getHelp()));

    //open text file
    openTextFileAction = new QAction(tr("Text File"), this);
    connect(openTextFileAction,SIGNAL(triggered()), this, SLOT(openTextFile()) );

    //open sdf file
    openSdfFileAction = new QAction(tr("Sdf File"), this);
    connect(openSdfFileAction,SIGNAL(triggered()), this, SLOT(openSdfFile()) );

    //export text file
    exportToTextAction = new QAction(tr("Text"),this);
    connect(exportToTextAction,SIGNAL(triggered()),this,SLOT(exportToTextFile()) );

    //export sdf file
    exportToSdfAction = new QAction(tr("SDF"),this);
    connect(exportToSdfAction,SIGNAL(triggered()),this,SLOT(exportToSdfFile()) );

    //make mpeg
    makeMpegAction = new QAction(tr("Mpeg"),this);
    //connection made in setupPlotWindow function

    //export as images
    exportImagesAction = new QAction(tr("Jpeg"),this);
    //connection made in setupPlotWindow function
}

//creates menus
void xvsMainWindow::createMenus()
{
    //creates menu option "File"
    fileMenu = menuBar()->addMenu(tr("&File"));
    openMenu = fileMenu->addMenu(tr("Open"));
    openMenu->addAction(openTextFileAction);
    openMenu->addAction(openSdfFileAction);
    exportMenu=fileMenu->addMenu(tr("Export to"));
    exportMenu->addAction(exportToSdfAction);
    exportMenu->addAction(exportToTextAction);
    exportMenu->addAction(makeMpegAction);
    exportMenu->addAction(exportImagesAction);
    fileMenu->addAction(getHelpAction);
    fileMenu->addAction(exitHandlerAction);

    //menu with several functions
    miscMenu = menuBar()->addMenu(tr("&Misc"));
    miscMenu->addAction(showControlDialogAction);
    miscMenu->addAction(set_insert_fuzz);
    miscMenu->addAction(reset_insert_fuzz);
    miscMenu->addAction(ctrl_space_kills_windows);
}

//help function- creates info dialog w/ url
void xvsMainWindow::getHelp()
{
    QMessageBox::StandardButton reply;
    reply= QMessageBox::information(this,"Help Resource",
                                    "For help/documentation see: http://laplace.physics.ubc.ca/Doc/xvs/");
}

/********************Misc Menu Actions********************/

//creates a line input dialog for new insert fuzz
void xvsMainWindow::getInsertFuzz()
{
    bool ok;
    double minValue = 0.00;
    double maxValue = 1e10;
    double d = QInputDialog::getDouble(this, tr("Input"),
                                       QString("Enter insert fuzz (current=%1)").arg(this->ifuzz),
                                       this->ifuzz,minValue,maxValue,5,&ok );
    if (ok)
        ifuzz = d;
}

//restores default insert fuzz (1e-10)
void xvsMainWindow::ResetInsertFuzz()
{
    ifuzz = 1e-10;
}

//activates/deactivates keyboard shortcut: Ctrl-space kills windows
void xvsMainWindow::toggle_ctrl_space_kills_windows(bool cskw)
{
    this->killActiveWindow->setEnabled(cskw);
    statusBar()->showMessage(QString("ctrl-space kills windows: %1").arg(cskw));
}

/********************end Misc Menu Actions********************/

/************************************************************/
/*******************END MENU BAR SECTION*********************/
/************************************************************/




/********************************************************************/
/*********************CONTROL DIALOG SECTION*************************/
/********************************************************************/
//creates dialog "Are you sure you want to exit?" on close statements
void xvsMainWindow::closeEvent(QCloseEvent *event)
{
    QMessageBox msgBox(QMessageBox::Warning, tr("Exit?"),
                       "Are you sure you want to exit?", 0, this);
    msgBox.addButton(tr("&Yes"), QMessageBox::AcceptRole);
    msgBox.addButton(tr("&No"), QMessageBox::RejectRole);
    if (msgBox.exec() == QMessageBox::AcceptRole) {event->accept();}
    else {event->ignore();}
}

//changes all checkboxes to same value
void xvsMainWindow::setAllCheckBoxState(bool check)
{
    emit allButtonToggled(check);
    statusBar()->showMessage(QString("Select All property: %1").arg(check),2000);
    if (allCheckBoxState != check)
    {
        allCheckBoxState = check;
    }
    if (allButton->isChecked() != allCheckBoxState)
    {
        allButton->setChecked(allCheckBoxState);
    }
    if (controlDialog->markerApplyAllCheckBox->isChecked() != allCheckBoxState)
    {
        controlDialog->markerApplyAllCheckBox->setChecked(allCheckBoxState);
    }
    if (controlDialog->allViewCheckBox->isChecked() != allCheckBoxState)
    {
        controlDialog->allViewCheckBox->setChecked(allCheckBoxState);
    }
    if (toggle_apply_all_on->isChecked() != allCheckBoxState)
    {
        toggle_apply_all_on->setChecked(allCheckBoxState);
    }
}

//Sets up connections from Control dialog box (only occurs in mainWindow contructor)
void xvsMainWindow::setupControlDialog()
{
    //Sets up link between "All" check boxes
    connect(allButton,SIGNAL(toggled(bool)),this,SLOT(setAllCheckBoxState(bool)));
    connect(controlDialog->allViewCheckBox,SIGNAL(toggled(bool)),this,SLOT(setAllCheckBoxState(bool)) );
    connect(controlDialog->markerApplyAllCheckBox, SIGNAL(toggled(bool)), this, SLOT(setAllCheckBoxState(bool)) );
    //connect(controlDialog, SIGNAL(), this, SLOT())
}

//connections common to all plot windows
void xvsMainWindow::setupPlotWindow(PlotWidget *w)
{
    //main window connections
    connect(this,SIGNAL(selectionModeActivated(int)),w,SLOT(changeDisplayMode(int)) );
    connect(w,SIGNAL(widgetSelected(int)),this,SLOT(addSelectedWidget(int)));
    connect(w,SIGNAL(widgetDeselected(int)),this,SLOT(removeSelectedWidget(int)));
    connect(w,SIGNAL(needUpdateControlDialog(QString)),this->formulaLabel,SLOT(setText(QString)));

    //view tab connections
    connect(controlDialog->modeComboBox, SIGNAL(activated(int)), w, SLOT(changeDisplayMode(int)) );
    connect(controlDialog->directionComboBox, SIGNAL(activated(int)), w, SLOT(changeTimeDirection(int)) );
    connect(controlDialog->breakComboBox, SIGNAL(activated(int)), w, SLOT(changeBreakStatus(int)));
    connect(controlDialog->opsIndexLine, SIGNAL(valueChanged(int)), w, SLOT(setTimeIndex(int)));
    connect(controlDialog->opsNameLine, SIGNAL(textEdited(QString)),w,SLOT(changeName(QString)));

    //marker tab connections
    connect(controlDialog->lineColorBox, SIGNAL(activated(int)), w, SLOT(changeLineColor(int)) );
    connect(controlDialog->markerColorBox, SIGNAL(activated(int)), w, SLOT(changeMarkerColor(int)) );
    connect(controlDialog->markerTypeInput, SIGNAL(activated(int)), w, SLOT(changeMarkerType(int)) );
    connect(controlDialog->pointSizeInput, SIGNAL(valueChanged(int)), w, SLOT(changeMarkerSize(int)) );
    connect(controlDialog->lineWidthInput, SIGNAL(valueChanged(int)), w, SLOT(changeLineWidth(int)) );
    connect(controlDialog->displayTypeInput, SIGNAL(currentIndexChanged(int)), w, SLOT(displayTypeHandler(int)) );

    //label tab connections
    connect(controlDialog, SIGNAL(labelToggled(int,bool)), w,SLOT(changeLabelShown(int,bool)));
    connect(controlDialog, SIGNAL(labelColorChanged(int,int)), w, SLOT(changeLabelColor(int,int)) );
    connect(controlDialog, SIGNAL(labelFormatChanged(int,int)), w, SLOT(changeLabelFormat(int,int)) );

    //view tab connections
    connect(controlDialog, SIGNAL(pushOntoStackSignal(plotLimits)),w,SLOT(resetPlotLimits(plotLimits)));
    connect(controlDialog->pushOntoZoomStackButton, SIGNAL(clicked()), w, SLOT(forward()));
    connect(controlDialog->popButton,SIGNAL(clicked()),w,SLOT(back()));
    connect(controlDialog->rescaleButton, SIGNAL(clicked()), w, SLOT(optimizePlotBoundaries()) );
    connect(controlDialog->viewIndexLine, SIGNAL(valueChanged(int)), w, SLOT(setTimeIndex(int)));
    connect(controlDialog->viewNameLine, SIGNAL(textEdited(QString)),w,SLOT(changeName(QString)));
    connect(this,SIGNAL(allButtonToggled(bool)),w,SLOT(setApplyAll(bool)));
    connect(controlDialog->fullScaleComboBox, SIGNAL(activated(int)), w, SLOT(toggleFullScale(int)));

    //browse data tab connections
    connect(controlDialog->browseIndexLine, SIGNAL(valueChanged(int)), w, SLOT(setTimeIndex(int)));
    connect(controlDialog->browseNameLine, SIGNAL(textEdited(QString)),w,SLOT(changeName(QString)));

    //mpeg connections
    connect(makeMpegAction, SIGNAL(triggered()),w,SLOT(makeMpeg()) );

    //image export connection
    connect(exportImagesAction,SIGNAL(triggered()),w,SLOT(exportToImage()) );

    //value update connections
    connect(w, SIGNAL(needUpdateControlDialog(QString)), this,SLOT(updateControlDialogValues(QString)) );
    updateControlDialogValues("updateAll");

    connect(w,SIGNAL(statusUpdate(QString)),this->statusBar(),SLOT(showMessage(QString)));
}

//resets the dialogue to reflect new active plot values
//current choices: updateAll
//                 view
//                 index
//                 name
//                 mode
void xvsMainWindow::updateControlDialogValues(QString s)
{
    PlotWidget *w = arrayPlotWidgets[activeWidgetIndex];
    QString str;

    int t = w->getTimeIndex();
    //set validator limits for index lines
    if (s=="updateAll"){
        if (w->curveMap.length() >0) controlDialog->updateData(w->curveMap.at(0));
        else controlDialog->updateData(w->data);
        int m = w->getMaxTimeIndex();
        controlDialog->opsIndexLine->setMaximum(m);
        controlDialog->viewIndexLine->setMaximum(m);
        controlDialog->browseIndexLine->setMaximum(m);
    }

    //updates view tab
    if ((s== "view") || (s=="updateAll"))
    {
        //resets line inputs xy min/maxes
        plotLimits pLims = w->getPlotLimits();
        controlDialog->xMinInput->setText(str.setNum(pLims.xMin));
        controlDialog->xMaxInput->setText(str.setNum(pLims.xMax));
        controlDialog->yMinInput->setText(str.setNum(pLims.yMin));
        controlDialog->yMaxInput->setText(str.setNum(pLims.yMax));

        //set up list view of zoomstack here
        QMap<int,QVector<double> >data = w->getZoomStack();
        controlDialog->stackListWidget->clear();
        for (int j=data.count()-2;j>=0;j--) //peculiar indexing a result of bad transfer format.
        {                                   //last element of data == (current zoom index)
            str = QString("xmin:%1, xmax:%2, ymin:%3,ymax:%4").arg(data[j].at(0)).arg(data[j].at(1)).arg(data[j].at(2)).arg(data[j].at(3));

            if (j == data[data.count()-1].at(0)) {
                controlDialog->stackListWidget->setCurrentItem(new QListWidgetItem(str,controlDialog->stackListWidget) );
            }
            else {
                new QListWidgetItem(str,controlDialog->stackListWidget);
            }
        }
        controlDialog->fullScaleComboBox->setCurrentIndex(w->getFullScale());
    }//end update view

    //time index updates
    if ((s== "index") || (s=="updateAll")){
        controlDialog->viewIndexLine->setValue(t);
        controlDialog->opsIndexLine->setValue(t);
        controlDialog->browseIndexLine->setValue(t);
        controlDialog->updateDataIndex(t);
        controlDialog->showCurrentData();
    }

    //name (plot title) update
    if ((s== "name") || (s=="updateAll")){
        controlDialog->viewNameLine->setText(w->getName());
        controlDialog->opsNameLine->setText(w->getName());
        controlDialog->browseNameLine->setText(w->getName());
    }

    //mode updates (as a result of hotkey functions)
    if ((s=="mode") || (s=="updateAll")) {
        if (w->getDisplayMode() != SELECTION) controlDialog->modeComboBox->setCurrentIndex(w->getDisplayMode());
        controlDialog->breakComboBox->setCurrentIndex(w->getBreakStatus());
        controlDialog->directionComboBox->setCurrentIndex(w->getTimeDirection());
    }

}

//shows or returns focus to control dialog
void xvsMainWindow::getControlDialog()
{
    controlDialog->show();
    controlDialog->activateWindow();
    if (this->numPlotWidgets >0) {
        updateControlDialogValues("updateAll");
    }
}
/********************************************************************/
/*********************END CONTROL DIALOG SECTION*********************/
/********************************************************************/




/************************EXIT CONTROL****************/
//Handles exit routine
void xvsMainWindow::exitHandeler()
{
    if (okToClose()){
        this->close();
    }
    if (okToQuit()){
        emit closeWindow();
    }
}

//Hack to stop menu toggling when undesired
bool xvsMainWindow::okToClose()
{
    if (!controlDialog || controlDialog->isHidden()){
        return true;
    }
    else{
        return true;
    }
}

//Ensures all windows are closed before quit() is invoked
bool xvsMainWindow::okToQuit()
{
    if ( (controlDialog->isHidden())&&(this->isHidden()) ){
        return true;
    }
    else{
        return false;
    }
}
/*******************END EXIT CONTROL*****************/


